# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class PaymentDisputeStatus(str, Enum):
    """
    PaymentDisputeStatus
    """

    """
    allowed enum values
    """
    PRE_WARNING_NEEDS_RESPONSE = 'pre_warning_needs_response'
    WARNING_NEEDS_RESPONSE = 'warning_needs_response'
    WARNING_NEEDS_REFUND = 'warning_needs_refund'
    WARNING_UNDER_REVIEW = 'warning_under_review'
    NEEDS_RESPONSE = 'needs_response'
    UNDER_REVIEW = 'under_review'
    LOST = 'lost'
    CANCEL = 'cancel'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PaymentDisputeStatus from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from payjpv2.models.setup_flow_list_response import SetupFlowListResponse

class TestSetupFlowListResponse(unittest.TestCase):
    """SetupFlowListResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SetupFlowListResponse:
        """Test SetupFlowListResponse
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SetupFlowListResponse`
        """
        model = SetupFlowListResponse()
        if include_optional:
            return SetupFlowListResponse(
                object = 'list',
                url = '',
                has_more = True,
                data = [
                    payjpv2.models.setup_flow_response.SetupFlowResponse(
                        object = 'setup_flow', 
                        id = '', 
                        livemode = True, 
                        client_secret = '', 
                        customer_id = '', 
                        description = '', 
                        metadata = {
                            'key' : null
                            }, 
                        payment_method_id = '', 
                        payment_method_options = { }, 
                        payment_method_types = [
                            'card'
                            ], 
                        status = 'canceled', 
                        next_action = { }, 
                        return_url = '', 
                        last_setup_error = { }, 
                        cancellation_reason = 'abandoned', 
                        created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                    ]
            )
        else:
            return SetupFlowListResponse(
                url = '',
                has_more = True,
                data = [
                    payjpv2.models.setup_flow_response.SetupFlowResponse(
                        object = 'setup_flow', 
                        id = '', 
                        livemode = True, 
                        client_secret = '', 
                        customer_id = '', 
                        description = '', 
                        metadata = {
                            'key' : null
                            }, 
                        payment_method_id = '', 
                        payment_method_options = { }, 
                        payment_method_types = [
                            'card'
                            ], 
                        status = 'canceled', 
                        next_action = { }, 
                        return_url = '', 
                        last_setup_error = { }, 
                        cancellation_reason = 'abandoned', 
                        created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                    ],
        )
        """

    def testSetupFlowListResponse(self):
        """Test SetupFlowListResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from payjpv2.models.payment_method_configuration_details_response import PaymentMethodConfigurationDetailsResponse

class TestPaymentMethodConfigurationDetailsResponse(unittest.TestCase):
    """PaymentMethodConfigurationDetailsResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PaymentMethodConfigurationDetailsResponse:
        """Test PaymentMethodConfigurationDetailsResponse
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PaymentMethodConfigurationDetailsResponse`
        """
        model = PaymentMethodConfigurationDetailsResponse()
        if include_optional:
            return PaymentMethodConfigurationDetailsResponse(
                object = 'payment_method_configuration',
                id = '',
                livemode = True,
                active = True,
                name = '',
                paypay = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                    available = True, 
                    display_preference = payjpv2.models.payment_method_configuration_display_preference.PaymentMethodConfigurationDisplayPreference(
                        preference = 'none', 
                        value = 'off', ), ),
                card = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                    available = True, 
                    display_preference = payjpv2.models.payment_method_configuration_display_preference.PaymentMethodConfigurationDisplayPreference(
                        preference = 'none', 
                        value = 'off', ), ),
                apple_pay = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                    available = True, 
                    display_preference = payjpv2.models.payment_method_configuration_display_preference.PaymentMethodConfigurationDisplayPreference(
                        preference = 'none', 
                        value = 'off', ), )
            )
        else:
            return PaymentMethodConfigurationDetailsResponse(
                id = '',
                livemode = True,
                active = True,
                name = '',
                paypay = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                    available = True, 
                    display_preference = payjpv2.models.payment_method_configuration_display_preference.PaymentMethodConfigurationDisplayPreference(
                        preference = 'none', 
                        value = 'off', ), ),
                card = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                    available = True, 
                    display_preference = payjpv2.models.payment_method_configuration_display_preference.PaymentMethodConfigurationDisplayPreference(
                        preference = 'none', 
                        value = 'off', ), ),
                apple_pay = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                    available = True, 
                    display_preference = payjpv2.models.payment_method_configuration_display_preference.PaymentMethodConfigurationDisplayPreference(
                        preference = 'none', 
                        value = 'off', ), ),
        )
        """

    def testPaymentMethodConfigurationDetailsResponse(self):
        """Test PaymentMethodConfigurationDetailsResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from payjpv2.api.setup_flows_api import SetupFlowsApi


class TestSetupFlowsApi(unittest.TestCase):
    """SetupFlowsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = SetupFlowsApi()

    def tearDown(self) -> None:
        pass

    def test_cancel_setup_flow(self) -> None:
        """Test case for cancel_setup_flow

        Cancel Setup Flow
        """
        pass

    def test_create_setup_flow(self) -> None:
        """Test case for create_setup_flow

        Create Setup Flow
        """
        pass

    def test_get_all_setup_flows(self) -> None:
        """Test case for get_all_setup_flows

        Get All Setup Flows
        """
        pass

    def test_get_setup_flow(self) -> None:
        """Test case for get_setup_flow

        Get Setup Flow
        """
        pass

    def test_update_setup_flow(self) -> None:
        """Test case for update_setup_flow

        Update Setup Flow
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from payjpv2.models.payment_method_configuration_setting_response import PaymentMethodConfigurationSettingResponse
from typing import Optional, Set
from typing_extensions import Self

class PaymentMethodConfigurationDetailsResponse(BaseModel):
    """
    PaymentMethodConfigurationDetailsResponse
    """ # noqa: E501
    object: Optional[StrictStr] = 'payment_method_configuration'
    id: StrictStr = Field(description="支払い方法設定 ID")
    livemode: StrictBool = Field(description="本番環境かどうか")
    active: StrictBool = Field(description="設定が有効かどうか")
    name: Optional[StrictStr]
    paypay: PaymentMethodConfigurationSettingResponse = Field(description="PayPay の設定")
    card: PaymentMethodConfigurationSettingResponse = Field(description="カードの設定")
    apple_pay: PaymentMethodConfigurationSettingResponse = Field(description="Apple Pay の設定")
    __properties: ClassVar[List[str]] = ["object", "id", "livemode", "active", "name", "paypay", "card", "apple_pay"]

    @field_validator('object')
    def object_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['payment_method_configuration']):
            raise ValueError("must be one of enum values ('payment_method_configuration')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.model_dump(by_alias=True, exclude_unset=True))

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PaymentMethodConfigurationDetailsResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of paypay
        if self.paypay:
            _dict['paypay'] = self.paypay.to_dict()
        # override the default output from pydantic by calling `to_dict()` of card
        if self.card:
            _dict['card'] = self.card.to_dict()
        # override the default output from pydantic by calling `to_dict()` of apple_pay
        if self.apple_pay:
            _dict['apple_pay'] = self.apple_pay.to_dict()
        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['name'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PaymentMethodConfigurationDetailsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "object": obj.get("object") if obj.get("object") is not None else 'payment_method_configuration',
            "id": obj.get("id"),
            "livemode": obj.get("livemode"),
            "active": obj.get("active"),
            "name": obj.get("name"),
            "paypay": PaymentMethodConfigurationSettingResponse.from_dict(obj["paypay"]) if obj.get("paypay") is not None else None,
            "card": PaymentMethodConfigurationSettingResponse.from_dict(obj["card"]) if obj.get("card") is not None else None,
            "apple_pay": PaymentMethodConfigurationSettingResponse.from_dict(obj["apple_pay"]) if obj.get("apple_pay") is not None else None
        })
        return _obj





# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from payjpv2.models.payment_method_configuration_list_response import PaymentMethodConfigurationListResponse

class TestPaymentMethodConfigurationListResponse(unittest.TestCase):
    """PaymentMethodConfigurationListResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PaymentMethodConfigurationListResponse:
        """Test PaymentMethodConfigurationListResponse
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PaymentMethodConfigurationListResponse`
        """
        model = PaymentMethodConfigurationListResponse()
        if include_optional:
            return PaymentMethodConfigurationListResponse(
                object = 'list',
                url = '',
                has_more = True,
                data = [
                    payjpv2.models.payment_method_configuration_details_response.PaymentMethodConfigurationDetailsResponse(
                        object = 'payment_method_configuration', 
                        id = '', 
                        livemode = True, 
                        active = True, 
                        name = '', 
                        paypay = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                            available = True, 
                            display_preference = payjpv2.models.payment_method_configuration_display_preference.PaymentMethodConfigurationDisplayPreference(
                                preference = 'none', 
                                value = 'off', ), ), 
                        card = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                            available = True, ), 
                        apple_pay = , )
                    ]
            )
        else:
            return PaymentMethodConfigurationListResponse(
                url = '',
                has_more = True,
                data = [
                    payjpv2.models.payment_method_configuration_details_response.PaymentMethodConfigurationDetailsResponse(
                        object = 'payment_method_configuration', 
                        id = '', 
                        livemode = True, 
                        active = True, 
                        name = '', 
                        paypay = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                            available = True, 
                            display_preference = payjpv2.models.payment_method_configuration_display_preference.PaymentMethodConfigurationDisplayPreference(
                                preference = 'none', 
                                value = 'off', ), ), 
                        card = payjpv2.models.payment_method_configuration_setting_response.PaymentMethodConfigurationSettingResponse(
                            available = True, ), 
                        apple_pay = , )
                    ],
        )
        """

    def testPaymentMethodConfigurationListResponse(self):
        """Test PaymentMethodConfigurationListResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



"""
Integration tests for PAY.JP Python SDK.

These tests verify the integration between different components of the SDK
without making actual API calls.
"""
import json
from unittest.mock import MagicMock, Mock, patch

import payjpv2
import pytest
from payjpv2.api.customers_api import CustomersApi
from payjpv2.api.payment_flows_api import PaymentFlowsApi
from payjpv2.api.payment_methods_api import PaymentMethodsApi
from payjpv2.models.customer_create_request import CustomerCreateRequest
from payjpv2.rest import ApiException

# from payjpv2.models.payment_flow_create_request import PaymentFlowCreateRequest
# from payjpv2.models.payment_method_create_request import PaymentMethodCreateRequest


class TestSDKIntegration:
    """Test SDK integration scenarios."""

    def test_client_initialization_with_api_key(self, api_key):
        """Test that the client can be initialized with an API key."""
        config = payjpv2.Configuration(
            host="https://api.pay.jp",
            api_key={'APIKeyHeader': api_key},
            api_key_prefix={'APIKeyHeader': 'Bearer'}
        )

        with payjpv2.ApiClient(config) as client:
            assert client.configuration.api_key['APIKeyHeader'] == api_key
            assert client.configuration.host == "https://api.pay.jp"
            assert client.configuration.api_key_prefix['APIKeyHeader'] == 'Bearer'

    def test_customer_workflow(self, api_client, sample_customer_data):
        """Test customer creation workflow."""
        customers_api = CustomersApi(api_client)

        # Mock the API call
        with patch.object(customers_api, 'create_customer') as mock_create:
            mock_response = Mock()
            mock_response.data = sample_customer_data
            mock_create.return_value = mock_response

            # Create customer request
            request = CustomerCreateRequest(email="test@example.com")
            response = customers_api.create_customer(request)

            # Verify the call was made correctly
            mock_create.assert_called_once_with(request)
            assert response.data['email'] == "test@example.com"

    def test_payment_flow_workflow(self, api_client, sample_payment_flow_data):
        """Test payment flow creation and confirmation workflow."""
        payment_flows_api = PaymentFlowsApi(api_client)

        with patch.object(payment_flows_api, 'get_payment_flow') as mock_retrieve:
            # Mock payment flow retrieval
            mock_response = Mock()
            mock_response.data = sample_payment_flow_data
            mock_retrieve.return_value = mock_response

            # Retrieve payment flow
            retrieved_pf = payment_flows_api.get_payment_flow("pfw_test123")

            # Verify workflow
            assert retrieved_pf.data['amount'] == 1000
            assert retrieved_pf.data['id'] == 'pfw_test123'

    def test_error_handling_integration(self, api_client):
        """Test error handling across API calls."""
        customers_api = CustomersApi(api_client)

        # Mock API exception
        with patch.object(customers_api, 'create_customer') as mock_create:
            mock_create.side_effect = ApiException(
                status=400,
                reason="Bad Request",
                body='{"error": {"message": "Invalid email format"}}'
            )

            request = CustomerCreateRequest(email="invalid-email")

            with pytest.raises(ApiException) as exc_info:
                customers_api.create_customer(request)

            assert exc_info.value.status == 400
            assert "Bad Request" in str(exc_info.value)

    def test_multiple_api_clients(self, api_key):
        """Test that multiple API clients can work independently."""
        config1 = payjpv2.Configuration(
            host="https://api.pay.jp",
            api_key={'APIKeyHeader': api_key + "_1"},
            api_key_prefix={'APIKeyHeader': 'Bearer'}
        )

        config2 = payjpv2.Configuration(
            host="https://api.pay.jp",
            api_key={'APIKeyHeader': api_key + "_2"},
            api_key_prefix={'APIKeyHeader': 'Bearer'}
        )

        with payjpv2.ApiClient(config1) as client1, \
             payjpv2.ApiClient(config2) as client2:

            customers_api1 = CustomersApi(client1)
            customers_api2 = CustomersApi(client2)

            # Verify they have different configurations
            assert client1.configuration.api_key['APIKeyHeader'] != client2.configuration.api_key['APIKeyHeader']
            assert customers_api1.api_client != customers_api2.api_client

    def test_api_client_context_manager(self, configuration):
        """Test API client context manager behavior."""
        with payjpv2.ApiClient(configuration) as client:
            assert client is not None
            assert client.configuration == configuration

            # Test that we can create API instances
            customers_api = CustomersApi(client)
            assert customers_api.api_client == client

    def test_http_request_configuration(self, api_client):
        """Test HTTP request configuration."""
        customers_api = CustomersApi(api_client)

        request = CustomerCreateRequest(email="test@example.com")

        with patch.object(customers_api.api_client, 'call_api') as mock_call_api:
            # Mock a simple response
            mock_response = Mock()
            mock_response.status = 200
            mock_response.data = b'{"id": "test"}'
            mock_response.headers = {}
            mock_call_api.return_value = mock_response

            with patch.object(customers_api.api_client, 'response_deserialize') as mock_deserialize:
                mock_deserialize.return_value = Mock(data={"id": "test"})
                customers_api.create_customer(request)

                # Verify call_api was called
                mock_call_api.assert_called_once()
                mock_deserialize.assert_called_once()


# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from payjpv2.models.payment_method_card_update_request import PaymentMethodCardUpdateRequest

class TestPaymentMethodCardUpdateRequest(unittest.TestCase):
    """PaymentMethodCardUpdateRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PaymentMethodCardUpdateRequest:
        """Test PaymentMethodCardUpdateRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PaymentMethodCardUpdateRequest`
        """
        model = PaymentMethodCardUpdateRequest()
        if include_optional:
            return PaymentMethodCardUpdateRequest(
                billing_details = payjpv2.models.payment_method_billing_details_request.PaymentMethodBillingDetailsRequest(
                    name = '', 
                    phone = '', 
                    email = '', 
                    address = payjpv2.models.payment_method_billing_address_request.PaymentMethodBillingAddressRequest(
                        country = '', 
                        zip = '', 
                        state = '', 
                        city = '', 
                        line1 = '', 
                        line2 = '', ), ),
                metadata = {
                    'key' : null
                    },
                type = 'card'
            )
        else:
            return PaymentMethodCardUpdateRequest(
                type = 'card',
        )
        """

    def testPaymentMethodCardUpdateRequest(self):
        """Test PaymentMethodCardUpdateRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class PaymentDisputeReason(str, Enum):
    """
    PaymentDisputeReason
    """

    """
    allowed enum values
    """
    RESEARCHING = 'researching'
    RESEARCHING_FRAUDULENT = 'researching_fraudulent'
    WARNED_FRAUDULENT = 'warned_fraudulent'
    CONFIRMED_FRAUDULENT = 'confirmed_fraudulent'
    UNRECOGNIZED = 'unrecognized'
    FRAUDULENT = 'fraudulent'
    PRODUCT_NOT_RECEIVED = 'product_not_received'
    ONLINE_FRAUDULENT = 'online_fraudulent'
    INCORRECT_ACCOUNT_DETAILS = 'incorrect_account_details'
    RECEIVING_CHARGEBACK = 'receiving_chargeback'
    DUPLICATE = 'duplicate'
    NOT_AUTHORIZED = 'not_authorized'
    SUBSCRIPTION_CANCELED = 'subscription_canceled'
    CHECK_RETURNED = 'check_returned'
    OTHER = 'other'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PaymentDisputeReason from a JSON string"""
        return cls(json.loads(json_str))





"""
Mock tests for PAY.JP API clients.

These tests use mocked responses to test API client behavior
without making actual network requests.
"""
import json
from unittest.mock import Mock, patch

import pytest
from payjpv2.api.customers_api import CustomersApi
from payjpv2.api.payment_flows_api import PaymentFlowsApi
from payjpv2.models.customer_create_request import CustomerCreateRequest
from payjpv2.models.payment_flow_create_request import PaymentFlowCreateRequest
from payjpv2.rest import ApiException


class TestCustomersApiMock:
    """Test Customers API with mocked responses."""

    @patch('payjpv2.api_client.ApiClient.response_deserialize')
    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_create_customer_success(self, mock_call_api, mock_deserialize, api_client, sample_customer_data):
        """Test successful customer creation."""
        customers_api = CustomersApi(api_client)

        # Mock the REST response
        mock_rest_response = Mock()
        mock_rest_response.status = 200
        mock_rest_response.data = json.dumps(sample_customer_data).encode()
        mock_rest_response.headers = {"Content-Type": "application/json"}
        mock_rest_response.read = Mock()
        mock_call_api.return_value = mock_rest_response

        # Mock the deserialization to return an object with data attribute
        mock_api_response = Mock()
        mock_api_response.data = sample_customer_data
        mock_deserialize.return_value = mock_api_response

        request = CustomerCreateRequest(email="test@example.com")
        response = customers_api.create_customer(request)

        # Verify API was called correctly
        mock_call_api.assert_called_once()
        mock_deserialize.assert_called_once()

        # Verify response
        assert response['id'] == 'cus_test123'
        assert response['email'] == 'test@example.com'

    @patch('payjpv2.api_client.ApiClient.response_deserialize')
    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_get_customer_success(self, mock_call_api, mock_deserialize, api_client, sample_customer_data):
        """Test successful customer retrieval."""
        customers_api = CustomersApi(api_client)

        mock_rest_response = Mock()
        mock_rest_response.status = 200
        mock_rest_response.data = json.dumps(sample_customer_data).encode()
        mock_call_api.return_value = mock_rest_response

        mock_api_response = Mock()
        mock_api_response.data = sample_customer_data
        mock_deserialize.return_value = mock_api_response

        response = customers_api.get_customer("cus_test123")

        # Verify API call
        mock_call_api.assert_called_once()
        mock_deserialize.assert_called_once()

        assert response['id'] == 'cus_test123'

    @patch('payjpv2.api_client.ApiClient.response_deserialize')
    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_list_customers_success(self, mock_call_api, mock_deserialize, api_client):
        """Test successful customer listing."""
        customers_api = CustomersApi(api_client)

        mock_list_data = {
            "object": "list",
            "data": [
                {"id": "cus_1", "email": "user1@example.com"},
                {"id": "cus_2", "email": "user2@example.com"}
            ],
            "has_more": False
        }

        mock_rest_response = Mock()
        mock_rest_response.status = 200
        mock_rest_response.data = json.dumps(mock_list_data).encode()
        mock_call_api.return_value = mock_rest_response

        mock_api_response = Mock()
        mock_api_response.data = mock_list_data
        mock_deserialize.return_value = mock_api_response

        response = customers_api.get_all_customers(limit=10)

        # Verify API call
        mock_call_api.assert_called_once()
        mock_deserialize.assert_called_once()

        assert response['object'] == 'list'
        assert len(response['data']) == 2


class TestPaymentFlowsApiMock:
    """Test Payment Flows API with mocked responses."""

    @patch('payjpv2.api_client.ApiClient.response_deserialize')
    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_create_payment_flow_success(self, mock_call_api, mock_deserialize, api_client, sample_payment_flow_data):
        """Test successful payment flow creation."""
        payment_flows_api = PaymentFlowsApi(api_client)

        mock_rest_response = Mock()
        mock_rest_response.status = 200
        mock_rest_response.data = json.dumps(sample_payment_flow_data).encode()
        mock_rest_response.read = Mock()
        mock_call_api.return_value = mock_rest_response

        mock_api_response = Mock()
        mock_api_response.data = sample_payment_flow_data
        mock_deserialize.return_value = mock_api_response

        request = PaymentFlowCreateRequest(amount=1000, currency="jpy")
        response = payment_flows_api.create_payment_flow(request)

        # Verify API call
        mock_call_api.assert_called_once()
        mock_deserialize.assert_called_once()

        assert response['amount'] == 1000

    @patch('payjpv2.api_client.ApiClient.response_deserialize')
    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_get_payment_flow_success(self, mock_call_api, mock_deserialize, api_client, sample_payment_flow_data):
        """Test successful payment flow retrieval."""
        payment_flows_api = PaymentFlowsApi(api_client)

        mock_rest_response = Mock()
        mock_rest_response.status = 200
        mock_rest_response.data = json.dumps(sample_payment_flow_data).encode()
        mock_rest_response.read = Mock()
        mock_call_api.return_value = mock_rest_response

        mock_api_response = Mock()
        mock_api_response.data = sample_payment_flow_data
        mock_deserialize.return_value = mock_api_response

        response = payment_flows_api.get_payment_flow("pfw_test123")

        # Verify API call
        mock_call_api.assert_called_once()
        mock_deserialize.assert_called_once()

        assert response['id'] == 'pfw_test123'


class TestApiErrorHandling:
    """Test API error handling with mocks."""

    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_api_exception_handling(self, mock_call_api, api_client):
        """Test API exception handling."""
        customers_api = CustomersApi(api_client)

        # Mock API exception
        mock_call_api.side_effect = ApiException(
            status=400,
            reason="Bad Request",
            body='{"error": {"message": "Invalid email"}}'
        )

        request = CustomerCreateRequest(email="invalid-email")

        with pytest.raises(ApiException) as exc_info:
            customers_api.create_customer(request)

        assert exc_info.value.status == 400
        assert exc_info.value.reason == "Bad Request"

    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_404_error_handling(self, mock_call_api, api_client):
        """Test 404 error handling."""
        customers_api = CustomersApi(api_client)

        mock_call_api.side_effect = ApiException(
            status=404,
            reason="Not Found",
            body='{"error": {"message": "Customer not found"}}'
        )

        with pytest.raises(ApiException) as exc_info:
            customers_api.get_customer("cus_nonexistent")

        assert exc_info.value.status == 404


class TestApiRequestBuilding:
    """Test API request building and parameter handling."""

    @patch('payjpv2.api_client.ApiClient.response_deserialize')
    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_query_parameters(self, mock_call_api, mock_deserialize, api_client):
        """Test query parameter handling."""
        customers_api = CustomersApi(api_client)

        mock_rest_response = Mock()
        mock_rest_response.status = 200
        mock_rest_response.data = b'{"object": "list", "data": []}'
        mock_rest_response.read = Mock()
        mock_call_api.return_value = mock_rest_response

        mock_api_response = Mock()
        mock_api_response.data = {"object": "list", "data": []}
        mock_deserialize.return_value = mock_api_response

        # Call with query parameters (use only valid parameters)
        customers_api.get_all_customers(limit=20)

        # Verify call_api was called
        mock_call_api.assert_called_once()
        mock_deserialize.assert_called_once()

    @patch('payjpv2.api_client.ApiClient.response_deserialize')
    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_request_body_serialization(self, mock_call_api, mock_deserialize, api_client):
        """Test request body serialization."""
        customers_api = CustomersApi(api_client)

        mock_rest_response = Mock()
        mock_rest_response.status = 200
        mock_rest_response.data = b'{"id": "cus_test"}'
        mock_rest_response.read = Mock()
        mock_call_api.return_value = mock_rest_response

        mock_api_response = Mock()
        mock_api_response.data = {"id": "cus_test"}
        mock_deserialize.return_value = mock_api_response

        request = CustomerCreateRequest(email="test@example.com")
        customers_api.create_customer(request)

        # Verify request body was included in the call
        mock_call_api.assert_called_once()
        mock_deserialize.assert_called_once()

    @patch('payjpv2.api_client.ApiClient.response_deserialize')
    @patch('payjpv2.api_client.ApiClient.call_api')
    def test_headers_configuration(self, mock_call_api, mock_deserialize, api_client):
        """Test HTTP headers configuration."""
        customers_api = CustomersApi(api_client)

        mock_rest_response = Mock()
        mock_rest_response.status = 200
        mock_rest_response.data = b'{"id": "cus_test"}'
        mock_rest_response.read = Mock()
        mock_call_api.return_value = mock_rest_response

        mock_api_response = Mock()
        mock_api_response.data = {"id": "cus_test"}
        mock_deserialize.return_value = mock_api_response

        request = CustomerCreateRequest(email="test@example.com")
        customers_api.create_customer(request)

        # Verify headers were configured (call_api was called)
        mock_call_api.assert_called_once()
        mock_deserialize.assert_called_once()


class TestModelIntegration:
    """Test model integration with API calls."""

    def test_customer_model_creation(self):
        """Test customer model creation."""
        request = CustomerCreateRequest(
            email="test@example.com",
            description="Test customer for API integration"
        )

        assert request.email == "test@example.com"
        assert request.description == "Test customer for API integration"

    def test_payment_flow_model_creation(self):
        """Test payment flow model creation."""
        request = PaymentFlowCreateRequest(
            amount=1000,
            currency="jpy",
            customer_id="cus_test123",
            description="Test payment flow"
        )

        assert request.amount == 1000
        assert request.customer_id == "cus_test123"
        assert request.description == "Test payment flow"

    def test_model_serialization_integration(self):
        """Test model serialization for API integration."""
        request = CustomerCreateRequest(email="test@example.com")

        # Test that model can be serialized for API calls
        json_str = request.to_json()
        assert isinstance(json_str, str)
        assert "test@example.com" in json_str


# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class StatementType(str, Enum):
    """
    Statementのタイプ  Statement自体が大枠としてどのような取引なのかを表す。 加盟店、PAYのオペレーター、システムがBalanceの状態を把握するときに利用できる分類にする。  ### 例 + type=sales     + 売上が発生している     + termを見ればどの区間の売上かがわかる + type=transfer_feeが2つ付いている     + 振込が2回行われた＝振込エラーが発生していた + type=forfeitが付いている     + この残高は失効した
    """

    """
    allowed enum values
    """
    SALES = 'sales'
    SERVICE_FEE = 'service_fee'
    TRANSFER_FEE = 'transfer_fee'
    FORFEIT = 'forfeit'
    MISC = 'misc'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of StatementType from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from payjpv2.api.payment_refunds_api import PaymentRefundsApi


class TestPaymentRefundsApi(unittest.TestCase):
    """PaymentRefundsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = PaymentRefundsApi()

    def tearDown(self) -> None:
        pass

    def test_create_payment_refund(self) -> None:
        """Test case for create_payment_refund

        Create Payment Refund
        """
        pass

    def test_get_all_payment_refunds(self) -> None:
        """Test case for get_all_payment_refunds

        Get All Payment Refunds
        """
        pass

    def test_get_payment_refund(self) -> None:
        """Test case for get_payment_refund

        Get Payment Refund
        """
        pass

    def test_update_payment_refund(self) -> None:
        """Test case for update_payment_refund

        Update Payment Refund
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from payjpv2.models.metadata_value import MetadataValue
from payjpv2.models.setup_flow_payment_method_options_request import SetupFlowPaymentMethodOptionsRequest
from payjpv2.models.usage import Usage
from typing import Optional, Set
from typing_extensions import Self

class SetupFlowCreateRequest(BaseModel):
    """
    SetupFlowCreateRequest
    """ # noqa: E501
    customer_id: Optional[StrictStr] = Field(default=None, description="この SetupFlow に関連付ける顧客の ID。SetupFlow により作られた PaymentMethod はこの顧客に紐付きます。")
    payment_method_options: Optional[SetupFlowPaymentMethodOptionsRequest] = Field(default=None, description="この SetupFlow 固有の支払い方法の設定")
    payment_method_types: Optional[List[StrictStr]] = Field(default=None, description="この SetupFlow で使用できる支払い方法の種類のリスト。 指定しない場合は、PAY.JP は支払い方法の設定から利用可能な支払い方法を動的に表示します。")
    usage: Optional[Usage] = Field(default=None, description="支払い方法が今後どのように使用されるかを指定します。指定されていない場合、この値はデフォルトで `off_session` になります。  | 値 | |:---| | **off_session**: 定期課金など、顧客がカートなどの決済フローにいるかどうか不明な場合は `off_session` を使用してください。 | | **on_session**: 顧客がカートなどの決済フローにいる場合にのみ支払い方法を利用する場合は `on_session` を使用してください。 |")
    description: Optional[StrictStr] = Field(default=None, description="説明。")
    metadata: Optional[Dict[str, MetadataValue]] = Field(default=None, description="キーバリューの任意のデータを格納できます。20件まで登録可能で、空文字列を指定するとそのキーを削除できます。<a href=\"https://docs.pay.jp/v2/guide/developers/metadata\">詳細はメタデータのドキュメントを参照してください。</a>")
    __properties: ClassVar[List[str]] = ["customer_id", "payment_method_options", "payment_method_types", "usage", "description", "metadata"]

    @field_validator('payment_method_types')
    def payment_method_types_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(['card']):
                raise ValueError("each list item must be one of ('card')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.model_dump(by_alias=True, exclude_unset=True))

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SetupFlowCreateRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of payment_method_options
        if self.payment_method_options:
            _dict['payment_method_options'] = self.payment_method_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each value in metadata (dict)
        _field_dict = {}
        if self.metadata:
            for _key_metadata in self.metadata:
                if self.metadata[_key_metadata]:
                    _field_dict[_key_metadata] = self.metadata[_key_metadata].to_dict()
            _dict['metadata'] = _field_dict
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SetupFlowCreateRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "customer_id": obj.get("customer_id"),
            "payment_method_options": SetupFlowPaymentMethodOptionsRequest.from_dict(obj["payment_method_options"]) if obj.get("payment_method_options") is not None else None,
            "payment_method_types": obj.get("payment_method_types"),
            "usage": obj.get("usage"),
            "description": obj.get("description"),
            "metadata": dict(
                (_k, MetadataValue.from_dict(_v))
                for _k, _v in obj["metadata"].items()
            )
            if obj.get("metadata") is not None
            else None
        })
        return _obj





"""
Tests for PAY.JP SDK model validation and serialization.
"""
import pytest
from payjpv2.models.customer_create_request import CustomerCreateRequest
from payjpv2.models.payment_flow_create_request import PaymentFlowCreateRequest


class TestCustomerModels:
    """Test customer-related models."""

    def test_customer_create_request_basic(self):
        """Test basic customer creation request."""
        request = CustomerCreateRequest()
        assert request is not None

    def test_customer_create_request_with_email(self):
        """Test customer creation request with email."""
        request = CustomerCreateRequest(email="test@example.com")
        assert request.email == "test@example.com"

    def test_customer_create_request_with_description(self):
        """Test customer creation with description."""
        request = CustomerCreateRequest(
            email="test@example.com",
            description="Test customer"
        )
        assert request.email == "test@example.com"
        assert request.description == "Test customer"

    def test_customer_create_request_minimal_valid(self):
        """Test that minimal customer request is valid."""
        request = CustomerCreateRequest()
        assert request is not None


class TestPaymentFlowModels:
    """Test payment flow-related models."""

    def test_payment_flow_create_request_basic(self):
        """Test basic payment flow creation request."""
        request = PaymentFlowCreateRequest(amount=1000, currency="jpy")
        assert request.amount == 1000

    def test_payment_flow_create_request_with_customer(self):
        """Test payment flow creation with customer."""
        request = PaymentFlowCreateRequest(
            amount=1000,
            currency="jpy",
            customer_id="cus_test123"
        )
        assert request.amount == 1000
        assert request.customer_id == "cus_test123"

    def test_payment_flow_create_request_with_description(self):
        """Test payment flow creation with description."""
        request = PaymentFlowCreateRequest(
            amount=1500,
            currency="jpy",
            description="Test payment for SDK"
        )
        assert request.amount == 1500
        assert request.description == "Test payment for SDK"

    def test_payment_flow_amount_types(self):
        """Test different amount values."""
        # Minimum valid amount (50)
        request1 = PaymentFlowCreateRequest(amount=50, currency="jpy")
        assert request1.amount == 50

        # Normal amount
        request2 = PaymentFlowCreateRequest(amount=1000, currency="jpy")
        assert request2.amount == 1000

        # Maximum valid amount (9999999)
        request3 = PaymentFlowCreateRequest(amount=9999999, currency="jpy")
        assert request3.amount == 9999999

    def test_payment_flow_capture_method(self):
        """Test capture method settings if available."""
        # Test that capture_method field exists and can be set
        request = PaymentFlowCreateRequest(amount=1000, currency="jpy")
        if hasattr(request, 'capture_method'):
            # Only test if the field exists in this implementation
            assert request.capture_method is None  # Default value


class TestModelSerialization:
    """Test model serialization capabilities."""

    def test_customer_request_serialization(self):
        """Test customer request serialization."""
        request = CustomerCreateRequest(
            email="test@example.com",
            description="Test customer"
        )

        # Test string representation
        str_repr = request.to_str()
        assert "test@example.com" in str_repr
        assert "Test customer" in str_repr

        # Test JSON serialization
        json_str = request.to_json()
        assert '"email": "test@example.com"' in json_str

    def test_payment_flow_request_serialization(self):
        """Test payment flow request serialization."""
        request = PaymentFlowCreateRequest(
            amount=1000,
            currency="jpy",
            description="Test payment"
        )

        str_repr = request.to_str()
        assert "1000" in str_repr
        assert "Test payment" in str_repr

        json_str = request.to_json()
        assert '"amount": 1000' in json_str

    def test_model_dict_conversion(self):
        """Test model to dict conversion."""
        request = CustomerCreateRequest(email="test@example.com")

        # Test dict conversion using model_dump if available
        if hasattr(request, 'model_dump'):
            data = request.model_dump()
            assert data["email"] == "test@example.com"


class TestModelValidation:
    """Test model validation flexibility."""

    def test_email_format_flexibility(self):
        """Test that email format is flexible at model level."""
        # These should all be accepted at model level
        emails = [
            "test@example.com",
            "test+tag@example.com",
            "test.user@example.co.jp",
            "invalid-email",  # Let server validate
            ""
        ]

        for email in emails:
            request = CustomerCreateRequest(email=email)
            assert request.email == email

    def test_amount_flexibility(self):
        """Test that amount validation works for valid values."""
        # PaymentFlowCreateRequest requires amount >= 50 and <= 9999999
        amounts = [50, 100, 1000, 9999999]

        for amount in amounts:
            request = PaymentFlowCreateRequest(amount=amount, currency="jpy")
            assert request.amount == amount

    def test_model_optional_fields(self):
        """Test that optional fields work correctly."""
        # Test customer with just email
        customer_request = CustomerCreateRequest(email="test@example.com")
        assert customer_request.email == "test@example.com"
        assert customer_request.description is None

        # Test payment flow with just amount and currency (required fields)
        payment_request = PaymentFlowCreateRequest(amount=1000, currency="jpy")
        assert payment_request.amount == 1000
        assert payment_request.customer_id is None


# coding: utf-8

"""
    PAY.JP API v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0.0
    Contact: support@pay.jp
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from payjpv2.api.payment_disputes_api import PaymentDisputesApi


class TestPaymentDisputesApi(unittest.TestCase):
    """PaymentDisputesApi unit test stubs"""

    def setUp(self) -> None:
        self.api = PaymentDisputesApi()

    def tearDown(self) -> None:
        pass

    def test_get_all_payment_disputes(self) -> None:
        """Test case for get_all_payment_disputes

        Get All Payment Disputes
        """
        pass

    def test_get_payment_dispute(self) -> None:
        """Test case for get_payment_dispute

        Get Payment Dispute
        """
        pass


if __name__ == '__main__':
    unittest.main()
